# Run daily by cron to download and apply modifications to geonames database.
# Output is sent (by cron) to '##PATH TO YOUR LOG FILE##/geonamesDaily.log'.

# Geonames download server changed from download.geonames.org to 
# api.geonames.org, Feb 2011.

# Job split into 2 steps, March 2012:
# step1: download the update files from geonames server
# step2: apply them
# Reason for split is that download is the step that sometimes fails (and we
# want to know asap) whereas step2 can be slow if lots of updates, so do it
# late in the evening.
 
# Amended to include timestamp on log entries, Nov 2012.

# THIS IS STEP 2.

# == Variables and functions ==================================================

MYSQLDIR=##LOCATION OF YOUR MYSQL DATABASE##
DAILYDIR=$MYSQLDIR/geonamesDailyFiles
ERRDIR=$MYSQLDIR/preservedUpdates
YESTERDAY=`date -d 'yesterday' +%Y-%m-%d`
DAYBEFOREYESTERDAY=`date -d '2 days ago' +%Y-%m-%d`
DOWNLOADERROR=""

# function called when SQL update returns an error
handleSqlError()
{
  mail -s "Geonames SQL update problem" k.byrne@ed.ac.uk, ##INSERT YOUR EMAIL ADDRESS(ES) HERE## <<EOF
An error occurred whilst updating the geonames MySQL database on ##YOUR HOST##.
Check ##PATH TO YOUR LOG FILE## for error messages.

The latest update files have been kept in case they need to be reapplied
later: ##PATH TO YOUR## preservedUpdates.
EOF
  # add note to log file too
  echo `date`": SQL error occurred when applying updates for $YESTERDAY."
  # preserve update files if errors occurred
  cp -p $DAILYDIR/* $ERRDIR
}

# -----------------------------------------------------------------------------

# == For each file (except hierarchy): load into holding tables, run updates, tidy up ==

# 1) -- geoname updates -------------------------------------------------------

if [ -e "$DAILYDIR/modifications-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  if [ -s "$DAILYDIR/modifications-$YESTERDAY.txt" ]
  then
    # file contains data (greater than 0 bytes)
    mysql --user=root --socket=/tmp/mysql.sock geonames <<EOF
truncate table updates_geoname;
load data local infile '$DAILYDIR/modifications-$YESTERDAY.txt' into table updates_geoname;
delete from geoname using geoname inner join updates_geoname where geoname.geonameId=updates_geoname.geonameId;
insert into geoname select * from updates_geoname;
truncate table updates_geoname;
EOF
    # NB changed from simpler syntax 
    # "delete from geoname where geonameid in (select geonameid from updates_geoname);"
    # because delete statement above is much faster in MySQL. Similarly for other updates.

    # check for SQL errors and send notification email if nec.
    if [ "$?" -gt "0" ]
    then
      handleSqlError
    fi
    echo `date`": `wc -l $DAILYDIR/modifications-$YESTERDAY.txt|awk '{print $1}'` updates to geoname table for $YESTERDAY."
  else
    echo `date`": No updates to geoname table for $YESTERDAY."
  fi
else
  # download problem - will already have been dealt with
  :
fi

# 2) -- geoname deletes -------------------------------------------------------

if [ -e "$DAILYDIR/deletes-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  if [ -s "$DAILYDIR/deletes-$YESTERDAY.txt" ]
  then
    #  file contains data (greater than 0 bytes)
    mysql --user=root --socket=/tmp/mysql.sock geonames <<EOF
truncate table deletes_geoname;
load data local infile '$DAILYDIR/deletes-$YESTERDAY.txt' into table deletes_geoname;
delete from geoname using geoname inner join deletes_geoname where geoname.geonameId=deletes_geoname.geonameId;
truncate table deletes_geoname;
EOF
    # check for SQL errors and send notification email if nec.
    if [ "$?" -gt "0" ]
    then
      handleSqlError
    fi
    echo `date`": `wc -l $DAILYDIR/deletes-$YESTERDAY.txt|awk '{print $1}'` deletes from geoname table for $YESTERDAY."
  else
    echo `date`": No deletes for geoname table for $YESTERDAY."
  fi
else
  # download problem - will already have been dealt with
  :
fi

# 3) -- alternatename updates -------------------------------------------------

if [ -e "$DAILYDIR/alternateNamesModifications-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  if [ -s "$DAILYDIR/alternateNamesModifications-$YESTERDAY.txt" ]
  then
    # file contains data (greater than 0 bytes)
    mysql --user=root --socket=/tmp/mysql.sock geonames <<EOF
truncate table updates_alternatename;
load data local infile '$DAILYDIR/alternateNamesModifications-$YESTERDAY.txt' into table updates_alternatename;
delete from alternatename using alternatename inner join updates_alternatename where alternatename.alternatenameId=updates_alternatename.alternatenameId;
insert into alternatename select * from updates_alternatename;
truncate table updates_alternatename;
EOF
    # check for SQL errors and send notification email if nec.
    if [ "$?" -gt "0" ]
    then
      handleSqlError
    fi
    echo `date`": `wc -l $DAILYDIR/alternateNamesModifications-$YESTERDAY.txt|awk '{print $1}'` updates to alternatename table for $YESTERDAY."
  else
    echo `date`": No updates to alternatename table for $YESTERDAY."
  fi
else
  # download problem - will already have been dealt with
  :
fi

# 4) -- alternatename deletes -------------------------------------------------

if [ -e "$DAILYDIR/alternateNamesDeletes-$YESTERDAY.txt" ]
then
  # file was successfully downloaded
  if [ -s "$DAILYDIR/alternateNamesDeletes-$YESTERDAY.txt" ]
  then
    # file contains data (greater than 0 bytes)
    mysql --user=root --socket=/tmp/mysql.sock geonames <<EOF
truncate table deletes_alternatename;
load data local infile '$DAILYDIR/alternateNamesDeletes-$YESTERDAY.txt' into table deletes_alternatename;
delete from alternatename using alternatename inner join deletes_alternatename where alternatename.geonameId=deletes_alternatename.geonameId;
truncate table deletes_alternatename;
EOF
    # check for SQL errors and send notification email if nec.
    if [ "$?" -gt "0" ]
    then
      handleSqlError
    fi
    echo `date`": `wc -l $DAILYDIR/alternateNamesDeletes-$YESTERDAY.txt|awk '{print $1}'` deletes from alternatename table for $YESTERDAY."
  else
    echo `date`": No deletes for alternatename table for $YESTERDAY."
  fi
else
  # download problem - will already have been dealt with
  :
fi

# --  Recreate hierarchy table each day -----------------------------------------

if [ -e "$DAILYDIR/hierarchy.zip" ]
then
  # file was successfully downloaded
  unzip $DAILYDIR/hierarchy.zip -d $DAILYDIR
  if [ -s "$DAILYDIR/hierarchy.txt" ]
  then
    # file unzipped ok and contains data (greater than 0 bytes)
    mysql --user=root --socket=/tmp/mysql.sock geonames <<EOF
drop table if exists hierarchy;
create table hierarchy (parentId int, childId int, type varchar(80)) character set utf8;
load data local infile '$DAILYDIR/hierarchy.txt' into table hierarchy;
create index parentInd on hierarchy(parentId);
create index childInd on hierarchy(childId);
EOF
    # check for SQL errors and send notification email if nec.
    if [ "$?" -gt "0" ]
    then
      handleSqlError
    fi
    echo `date`": `wc -l $DAILYDIR/hierarchy.txt|awk '{print $1}'` rows in hierachy table for $YESTERDAY."
  else
    echo `date`": Nothing in hierarchy download file for $YESTERDAY; table left unchanged."
  fi
else
  # download problem - will already have been dealt with
  :
fi

echo

#-----------------------------------------------------------------------------
